package com.example.controller;

import com.example.config.annotation.IgnoreAuth;
import com.example.entity.OtnAreaEntity;
import com.example.entity.WzHouseContractInEntity;
import com.example.entity.vo.contract.RentInVo;
import com.example.service.WzHouseContractInService;
import com.example.service.WzOtnAreaService;
import com.example.utils.PageMap;
import com.example.utils.PageUtils;
import com.example.utils.Query;
import com.example.utils.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 房屋出租(合同管理)
 */
@RestController
@RequestMapping("/house-car/house/dist/api/WzHouseContractIn")
public class WzHouseContractInController {

    @Autowired
    private WzOtnAreaService otnAreaService;
    @Autowired
    private WzHouseContractInService wzHouseContractInService;

    @RequestMapping("/list")
    @IgnoreAuth
    public R list(@RequestBody PageMap pageMap) {
        //查询列表数据
        Query query = new Query(pageMap);
        List<WzHouseContractInEntity> list = wzHouseContractInService.queryList(query);
        Query query1 = new Query(pageMap);
        int total = wzHouseContractInService.queryTotal(query1);
        PageUtils pageUtil = new PageUtils(list, total, query.getLimit(), query.getPage());
        return R.ok().put("page", pageUtil);
    }

    @RequestMapping("/echartList")
    @IgnoreAuth
    public R echartList(@RequestBody PageMap pageMap) {
        //查询列表数据
        Query query = new Query(pageMap);
        List<WzHouseContractInEntity> alllist = wzHouseContractInService.queryList(pageMap.getMap());

        //全省节点
        RentInVo province=new RentInVo();
        province.setCounty("全省");
        province.setCity("全省");

        int aContractNum=0;
        double aMoney=0l;
        double aAvage=0l;
        double aRentArea=0l;
        //按地市进行划分
        Map<String,List<WzHouseContractInEntity>> cityMap=new HashMap<>();
        for (WzHouseContractInEntity entity : alllist) {
            if(entity.getCity()==null) continue;
            List<WzHouseContractInEntity> list;
            if(cityMap.get(entity.getCity())!=null){
                list=cityMap.get(entity.getCity());
                list.add(entity);
            }else{
                list=new ArrayList<>();
                list.add(entity);
            }
            cityMap.put(entity.getCity(),list);
        }
        //按区进行划分
        List<RentInVo> resultList=new ArrayList<>();
        List<RentInVo> cityList=new ArrayList<>();
        for(Map.Entry<String,List<WzHouseContractInEntity>> entry:cityMap.entrySet()){
            List<WzHouseContractInEntity> list=entry.getValue();
            //县的Map集合
            Map<String,List<WzHouseContractInEntity>> countyMap=new HashMap<>();
            for (WzHouseContractInEntity leaveUseVo : list) {
                if(countyMap.get(leaveUseVo.getCounty())!=null){
                    list=countyMap.get(leaveUseVo.getCounty());
                    list.add(leaveUseVo);
                }else{
                    list=new ArrayList<>();
                    list.add(leaveUseVo);
                }
                countyMap.put(leaveUseVo.getCounty(),list);
            }

            List<RentInVo> countyList=new ArrayList<>();
            int cityContractNum=0;
            double cityMoney=0l;
            double cityRentArea=0l;
            double cityAverage=0l;
            for (Map.Entry<String,List<WzHouseContractInEntity>> countyEntry : countyMap.entrySet()) {
                int contractNum=0;
                double money=0l;
                double average=0l;
                double rentArea=0l;

                RentInVo county=new RentInVo();
                for (WzHouseContractInEntity leaveUseVo : countyEntry.getValue()) {
                    county.setCity(leaveUseVo.getCounty());
                    contractNum+=1;
                    rentArea+=leaveUseVo.getSelfUseBuildingArea();
                    money+=leaveUseVo.getContractAmountIncludingTax();

                    cityContractNum+=1;
                    cityRentArea+=leaveUseVo.getSelfUseBuildingArea();
                    cityMoney+=leaveUseVo.getContractAmountIncludingTax();

                    aContractNum+=1;
                    aRentArea+=leaveUseVo.getSelfUseBuildingArea();
                    aMoney+=leaveUseVo.getContractAmountIncludingTax();
                }
                county.setContractNum(contractNum);
                county.setRentArea(formatDouble(rentArea));
                county.setMoney(formatDouble(money));
                if(rentArea==0){
                    county.setAverage(0);
                }else{
                    county.setAverage(formatDouble(money/rentArea));
                }

                countyList.add(county);
            }

            List<RentInVo> orderCountyList=new ArrayList<>();
            try{
                //对countList 进行排序
                String city=list.get(0).getCity();
                if(city.contains("市")){
                    city=city.split("市")[0];
                }
                OtnAreaEntity area=otnAreaService.queryObject(city);
                List<OtnAreaEntity> countyAreaList=otnAreaService.getCityOption(area.getId());
                for (OtnAreaEntity otnAreaEntity : countyAreaList) {
                    for (RentInVo wzCity : countyList) {
                        if(wzCity.getCity().contains(otnAreaEntity.getName())){
                            orderCountyList.add(wzCity);
                        }
                    }
                }
            }catch (Exception e){
                orderCountyList=countyList;
                e.printStackTrace();
            }

            RentInVo parent=new RentInVo();
            parent.setContractNum(cityContractNum);
            parent.setMoney(formatDouble(cityMoney));
            parent.setCity(list.get(0).getCity());
            parent.setChildren(orderCountyList);
            parent.setRentArea(formatDouble(cityRentArea));
            if(cityRentArea==0){
                parent.setAverage(0);
            }else{
                parent.setAverage(formatDouble(cityMoney/cityRentArea));
            }
            cityList.add(parent);
        }

        //对cityList 进行排序
        List<OtnAreaEntity> areaList=otnAreaService.getCityOption("018");
        List<RentInVo> orderList=new ArrayList<>();
        for (OtnAreaEntity otnAreaEntity : areaList) {
            for (RentInVo wzCity : cityList) {
                if(wzCity.getCity().contains(otnAreaEntity.getName())){
                    orderList.add(wzCity);
                }
            }
        }

        province.setChildren(orderList);
        province.setMoney(formatDouble(aMoney));
        province.setContractNum(aContractNum);
        province.setAverage(formatDouble(aMoney/aRentArea));
        province.setRentArea(formatDouble(aRentArea));
        resultList.add(province);

        int total = wzHouseContractInService.queryTotal(query);
        PageUtils pageUtil = new PageUtils(resultList, total, query.getLimit(), query.getPage());
        return R.ok().put("page", pageUtil);
    }

    public double formatDouble(double num){
        double avera=0;
        try{
            DecimalFormat format2 = new DecimalFormat("#.00");
            String str2 = format2.format(num);
            avera = Double.parseDouble(str2);
        }catch (Exception e){
            e.printStackTrace();
        }
        return avera;
    }
}
