package com.example.pojo.vo;

import com.example.entity.house.HouseBuildingPo;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.math.BigDecimal;
import java.text.DecimalFormat;

@Data
public class ListBuildingIdleVo {
    @Schema(description = "账期", example = "202312")
    private Integer statisticalMonth;
    @Schema(description = "资产所属单位一级")
    private String province;
    @Schema(description = "资产所属单位二级")
    private String city;
    @Schema(description = "资产所属单位三级")
    private String county;
    @Schema(description = "建筑别名")
    private String buildingAias;
    @Schema(description = "建筑面积㎡")
    private BigDecimal buildingArea;
    @Schema(description = "建筑面积自用㎡")
    private BigDecimal buildingAreaUseArea;
    @Schema(description = "建筑面积出租㎡")
    private BigDecimal buildingAreaRentArea;
    @Schema(description = "建筑面积闲置㎡")
    private BigDecimal buildingAreaIdelArea;
    @Schema(description = "建筑面积不可使用㎡")
    private BigDecimal buildingAreaUnavailable;
    @Schema(description = "标准地址")
    private String standardAddress;
    @Schema(description = "建筑用途")
    private String buildingUse;
    private String buildingRate;            //得房率
    private String parentBuildingName;      //上级局址名称
    private String parentLandName;          //上级土地名称
    private String buildingSource;          //房屋来源
    private String getDate;                 //取得日期
    private Integer buildingStartYear;       //房龄开始年份
    private String investmentEntity;        //投资主体
    private String managementHierarchy;     //管理层级
    private String buildingStructure;       //房屋结构
    private String totalNumberOfFloors;     //楼层总数
    private String buildingFaceStreet;      //是否临街
    private String courtyard;               //是否有院落
    private String wholeBuilding;           //整栋是否独有
    private String premisesPermit;          //是否有房产证
    private String noPremisesPermitReason;  //无房产证原因
    private String unrelatedAssets;         //未关联资产
    private String assetCode;               //资产编号
    private String astsLabe;                //资产标签号
    private String useState;                //使用状态
    private String ownershipStatus;         //权属状态
    private BigDecimal buildingFloorArea;       //建筑占地面积(㎡)
    private BigDecimal useArea;                 //使用面积(㎡)
    private BigDecimal useAreaSelfUseArea;      //使用面积-自用(㎡)
    private BigDecimal useAreaRentArea;         //使用面积-出租(㎡)
    private BigDecimal useAreaIdelArea;         //使用面积-闲置(㎡)
    private BigDecimal useAreaUnavailableArea;  //使用面积-不可使用(㎡)
    private String landManageName;      //楼长姓名
    private String landManageCompany;   //楼长所在单位
    private BigDecimal buildingLongitude;       //房屋经度
    private BigDecimal buildingDimension;       //房屋纬度
    private String buildingActualPropertyRights;  //房屋实际产权
    private String buildingAddressNumber;   //局址编码
    private String buildingNameAlias;       //局址别名
    private String areaSector;

    public ListBuildingIdleVo(HouseBuildingPo po) {
        this.statisticalMonth = po.getYearMonth();
        this.province = po.getFirstUnit();
        this.city = po.getAreaName();
        this.county = po.getCityName();
        this.buildingAias = po.getBuildingName();
        this.buildingArea = po.getBuildingArea();
        this.buildingAreaUseArea = po.getBuildingAreaSelfUse();
        this.buildingAreaRentArea = po.getBuildingAreaRent();
        this.buildingAreaIdelArea = po.getBuildingAreaIdle();
        this.buildingAreaUnavailable = po.getBuildingAreaUnusable();
        this.standardAddress = po.getAddress();
        this.buildingUse = po.getBuildingUse();
        this.buildingRate = po.getHousingAcquisitionRate() != null
                ? new DecimalFormat("#.00%").format(po.getHousingAcquisitionRate()) : null;
        this.parentBuildingName = po.getSiteName();
        this.parentLandName = po.getLandName();
        this.buildingSource = po.getHousingSource();
        this.getDate = po.getAcquisitionDate();
        this.buildingStartYear = po.getHouseYearBegan();
        this.investmentEntity = po.getInvestor();
        this.managementHierarchy = po.getManagementLevel();
        this.buildingStructure = po.getBuildingStructure();
        this.totalNumberOfFloors = po.getTotalFloors();
        this.buildingFaceStreet = po.getFrontage();
        this.courtyard = po.getCourtyard();
        this.wholeBuilding = po.getWholeBuilding();
        this.premisesPermit = po.getPropertyOwnershipCertificate();
        this.noPremisesPermitReason = po.getNoPropertyOwnershipCertificateReason();
        this.unrelatedAssets = po.getUnrelatedAssets();
        this.assetCode = po.getAssetsNum();
        this.astsLabe = po.getAssetsTagNum();
        this.useState = po.getUsageStatus();
        this.ownershipStatus = po.getOwnershipStatus();
        this.buildingFloorArea = po.getFloorArea();
        this.useArea = po.getUsableArea();
        this.useAreaSelfUseArea = po.getUsableAreaSelfUse();
        this.useAreaRentArea = po.getUsableAreaRent();
        this.useAreaIdelArea = po.getUsableAreaIdle();
        this.useAreaUnavailableArea = po.getUsableAreaUnusable();
        this.landManageName = po.getCommunityAssistantName();
        this.landManageCompany = po.getCommunityAssistantUnit();
        this.buildingLongitude = po.getLngJt();
        this.buildingDimension = po.getLatJt();
        this.buildingActualPropertyRights = po.getPropertyOwner();
        this.buildingAddressNumber = po.getSiteNum();
        this.buildingNameAlias = po.getSiteName();
        this.areaSector = po.getAreaSector();
    }
}
