package com.example.service.house;

import com.example.common.Rsp;
import com.example.dao.house.HouseSiteStatDao;
import com.example.entity.house.HouseBuildingPo;
import com.example.entity.house.HouseSiteBuildingAreaIdleDiffPo;
import com.example.entity.house.HouseSitePo;
import com.example.entity.house.HouseSiteRepairInvestorStatPo;
import com.example.entity.house.HouseSiteRepairMonthPo;
import com.example.pojo.dto.GetSiteStatDto;
import com.example.pojo.vo.GetSiteStatBuildingVo;
import com.example.pojo.vo.GetSiteStatVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

@Slf4j
@Service
public class HouseSiteStatService {
    private final HouseSiteStatDao houseSiteStatDao;

    public HouseSiteStatService(HouseSiteStatDao houseSiteStatDao) {
        this.houseSiteStatDao = houseSiteStatDao;
    }

    public Rsp<GetSiteStatVo> getSiteStat(GetSiteStatDto dto) {
        GetSiteStatVo vo = getSiteStatVo(dto);
        return Rsp.ok(vo);
    }

    private GetSiteStatVo getSiteStatVo(GetSiteStatDto dto) {
        GetSiteStatVo vo = new GetSiteStatVo();
        Integer endYearMonth = dto.getMap().getStatisticalMonth();
        LocalDate localDateEnd = LocalDate.parse(endYearMonth + "01", DateTimeFormatter.ofPattern("yyyyMMdd"));
        LocalDate localDateStart = localDateEnd.withDayOfYear(1);
        Integer startYearMonth = Integer.valueOf(localDateStart.format(DateTimeFormatter.ofPattern("yyyyMM")));
        int endMonth = localDateEnd.getMonthValue();
        String siteId = dto.getMap().getSiteId();
        // 默认燕赵信息大厦
        if (!StringUtils.hasText(siteId)) {
            siteId = "2017042617021008131986";
        }
        // 查询局址信息
        HouseSitePo houseSitePo = houseSiteStatDao.getHouseSiteBySiteIdAndYearMonth(endYearMonth, siteId);
        vo.setSiteNum(houseSitePo.getSiteNum());
        vo.setSiteName(houseSitePo.getSiteName());
        vo.setAreaName(houseSitePo.getAreaName());
        vo.setCityName(houseSitePo.getCityName());
        vo.setAddress(houseSitePo.getAddress());
        vo.setAreaSector(houseSitePo.getAreaSector());
        // 查询局址下的所有建筑
        List<HouseBuildingPo> houseBuildingPoList = houseSiteStatDao.allBuildingBySiteIdAndYearMonth(endYearMonth,
                siteId);
        BigDecimal buildingAreaSum = BigDecimal.ZERO;
        BigDecimal buildingAreaSelfUseSum = BigDecimal.ZERO;
        BigDecimal buildingAreaRentSum = BigDecimal.ZERO;
        BigDecimal buildingAreaIdleSum = BigDecimal.ZERO;
        BigDecimal buildingAreaUnusableSum = BigDecimal.ZERO;
        List<GetSiteStatBuildingVo> buildingList = new ArrayList<>();
        vo.setBuildingList(buildingList);
        for (HouseBuildingPo houseBuildingPo : houseBuildingPoList) {
            buildingList.add(new GetSiteStatBuildingVo(houseBuildingPo));
            // 计算局址建筑面积
            if (houseBuildingPo.getBuildingArea() != null) {
                buildingAreaSum = buildingAreaSum.add(houseBuildingPo.getBuildingArea());
            }
            if (houseBuildingPo.getBuildingAreaSelfUse() != null) {
                buildingAreaSelfUseSum = buildingAreaSelfUseSum.add(houseBuildingPo.getBuildingAreaSelfUse());
            }
            if (houseBuildingPo.getBuildingAreaRent() != null) {
                buildingAreaRentSum = buildingAreaRentSum.add(houseBuildingPo.getBuildingAreaRent());
            }
            if (houseBuildingPo.getBuildingAreaIdle() != null) {
                buildingAreaIdleSum = buildingAreaIdleSum.add(houseBuildingPo.getBuildingAreaIdle());
            }
            if (houseBuildingPo.getBuildingAreaUnusable() != null) {
                buildingAreaUnusableSum = buildingAreaUnusableSum.add(houseBuildingPo.getBuildingAreaUnusable());
            }
        }
        vo.setBuildingCount(buildingList.size());
        vo.setBuildingAreaSum(buildingAreaSum);
        vo.setBuildingAreaSelfUseSum(buildingAreaSelfUseSum);
        vo.setBuildingAreaRentSum(buildingAreaRentSum);
        vo.setBuildingAreaIdleSum(buildingAreaIdleSum);
        vo.setBuildingAreaUnusableSum(buildingAreaUnusableSum);
        // 查询局址土地总面积
        BigDecimal landAreaSum = houseSiteStatDao.getLandAreaBySiteIdAndYearMonth(endYearMonth, siteId);
        vo.setLandAreaSum(landAreaSum == null ? BigDecimal.ZERO : landAreaSum);
        // 查询房屋维修费累计
        HouseSiteRepairInvestorStatPo houseSiteRepairInvestorStatPo
                = houseSiteStatDao.getSiteRepairInvestorStat(startYearMonth, endYearMonth, siteId);
        vo.setBuildingRepairSs(houseSiteRepairInvestorStatPo.getBuildingRepairSs());
        vo.setBuildingRepairCx(houseSiteRepairInvestorStatPo.getBuildingRepairCx());
        // 查询局址分月维修费
        List<HouseSiteRepairMonthPo> houseSiteRepairMonthPoList
                = houseSiteStatDao.getHouseSiteRepairMonth(startYearMonth, endYearMonth, siteId);
        LinkedHashMap<String, BigDecimal> buildingRepairMonth = new LinkedHashMap<>();
        vo.setBuildingRepairMonth(buildingRepairMonth);
        for (int i = 1; i <= endMonth; i++) {
            buildingRepairMonth.put(String.valueOf(i), BigDecimal.ZERO);
            for (HouseSiteRepairMonthPo houseSiteRepairMonthPo : houseSiteRepairMonthPoList) {
                if (houseSiteRepairMonthPo.getMonthNo().equals(i) && houseSiteRepairMonthPo.getFinalCostSum() != null) {
                    buildingRepairMonth.put(String.valueOf(i), houseSiteRepairMonthPo.getFinalCostSum());
                }
            }
        }
        // 查询局址房屋面积闲置率变化
        HouseSiteBuildingAreaIdleDiffPo houseSiteBuildingAreaIdleDiffPo
                = houseSiteStatDao.getSiteBuildingAreaIdleDiff(startYearMonth, endYearMonth, siteId);
        vo.setBuildingAreaIdleRatePast(houseSiteBuildingAreaIdleDiffPo.getBuildingAreaIdleRatePast());
        vo.setBuildingAreaIdleRateNow(houseSiteBuildingAreaIdleDiffPo.getBuildingAreaIdleRateNow());
        vo.setBuildingAreaIdleRateDiff(houseSiteBuildingAreaIdleDiffPo.getBuildingAreaIdleRateDiff());
        return vo;
    }
}
