package com.example.service.house;

import cn.hutool.core.net.URLEncodeUtil;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.common.PageVo;
import com.example.common.Rsp;
import com.example.dao.WzOtnAreaDao;
import com.example.dao.house.HouseBuildingDao;
import com.example.entity.house.HouseBuildingPo;
import com.example.entity.house.HouseSitePo;
import com.example.enums.ListBuildingIdleOrderEnum;
import com.example.enums.OrderEnum;
import com.example.pojo.bo.ListBuildingIdleBo;
import com.example.pojo.bo.ListHouseSiteBo;
import com.example.pojo.dto.ListBuildingIdleDto;
import com.example.pojo.dto.ListSiteNameDto;
import com.example.pojo.dto.ListSiteNumDto;
import com.example.pojo.vo.ListBuildingIdleVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

@Slf4j
@Service
public class HouseWzBuildManageDetailsService {
    private final HouseBuildingDao houseBuildingDao;
    private final WzOtnAreaDao wzOtnAreaDao;

    public HouseWzBuildManageDetailsService(HouseBuildingDao houseBuildingDao, WzOtnAreaDao wzOtnAreaDao) {
        this.houseBuildingDao = houseBuildingDao;
        this.wzOtnAreaDao = wzOtnAreaDao;
    }

    public Rsp<PageVo<ListBuildingIdleVo>> ideList(ListBuildingIdleDto dto) {
        Page<HouseBuildingPo> page = new Page<>(dto.getPage().getPageNum(), dto.getPage().getPageSize());
        List<HouseBuildingPo> list = getBuildingIdle(dto, page);
        List<ListBuildingIdleVo> vos = new ArrayList<>();
        for (HouseBuildingPo po : list) {
            vos.add(new ListBuildingIdleVo(po));
        }
        PageVo<ListBuildingIdleVo> pageVo = new PageVo<>(vos, page);
        return Rsp.ok(pageVo);
    }

    public void ideListExport(ListBuildingIdleDto dto) {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder
                .getRequestAttributes();
        if (servletRequestAttributes == null) {
            return;
        }
        HttpServletResponse response = servletRequestAttributes.getResponse();
        if (response == null) {
            return;
        }
        String filename = "房屋空置_空置1000平方米以上"
                + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMdd")) + ".xlsx";
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncodeUtil.encode(filename));
        response.setContentType("application/octet-stream");
        try (SXSSFWorkbook wb = new SXSSFWorkbook()) {
            writeIdleList(dto, wb);
            wb.write(response.getOutputStream());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    private List<HouseBuildingPo> getBuildingIdle(ListBuildingIdleDto dto, Page<HouseBuildingPo> page) {
        ListBuildingIdleBo bo = new ListBuildingIdleBo();
        if (dto.getMap() != null && dto.getMap().getStatisticalMonth() != null) {
            bo.setYearMonth(dto.getMap().getStatisticalMonth());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCity()) && !"全省".equals(dto.getMap().getCity())) {
            bo.setAreaNo(dto.getMap().getCity());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCounty())) {
            bo.setCityNo(dto.getMap().getCounty());
        }
        if (dto.getMap() != null && dto.getMap().getOrder() != null && dto.getMap().getSidx() != null) {
            if (OrderEnum.asc.equals(dto.getMap().getOrder())) {
                page.addOrder(OrderItem.asc(dto.getMap().getSidx().getColumnName()));
            } else {
                page.addOrder(OrderItem.desc(dto.getMap().getSidx().getColumnName()));
            }
        } else {
            page.addOrder(OrderItem.desc(ListBuildingIdleOrderEnum.buildingAreaIdelArea.getColumnName()));
        }
        return houseBuildingDao.listBuildingIdle(page, bo);
    }

    /**
     * 写空置1000平米以上建筑文件
     */
    private void writeIdleList(ListBuildingIdleDto dto, SXSSFWorkbook wb) {
        Page<HouseBuildingPo> page = new Page<>(1, Long.MAX_VALUE);
        List<HouseBuildingPo> list = getBuildingIdle(dto, page);
        DataFormat dataFormat = wb.createDataFormat();
        // 数字样式
        CellStyle numberCellStyle = wb.createCellStyle();
        numberCellStyle.setDataFormat(dataFormat.getFormat("#,##0.00"));
        numberCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        // 默认样式
        CellStyle baseCellStyle = wb.createCellStyle();
        baseCellStyle.setAlignment(HorizontalAlignment.CENTER);
        baseCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        CellStyle percentCellStyle = wb.createCellStyle();
        percentCellStyle.setDataFormat(dataFormat.getFormat("0.00%"));
        percentCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        SXSSFSheet sheet = wb.createSheet();
        AtomicInteger rowIndex = new AtomicInteger(0);
        // 表头
        Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
        List<String> headers = Stream.of("资产所属单位(一级)", "资产所属单位(二级)", "资产所属单位(三级)", "局址编号",
                "局址别名", "建筑别名", "地段", "建筑面积(㎡)","建筑面积-出租(㎡)", "建筑面积-闲置(㎡)", "建筑面积-自用(㎡)",
                "建筑面积-不可使用(㎡)", "上级土地名称", "得房率", "房屋来源", "取得日期", "房龄开始年份", "投资主体", "管理层级",
                "房屋结构", "楼层总数", "是否临街", "是否有院落", "整栋是否独有", "是否有房产证", "无房产证原因", "未关联资产",
                "资产编号", "资产标签号", "使用状态", "建筑用途", "权属状态", "建筑占地面积(㎡)", "使用面积(㎡)",
                "使用面积-自用(㎡)", "使用面积-出租(㎡)", "使用面积-闲置(㎡)", "使用面积-不可使用(㎡)", "楼长姓名",
                "楼长所在单位", "经度", "纬度", "实际产权人").toList();
        int headerLength = headers.size();
        for (int i = 0; i < headerLength; i++) {
            Cell cell = headerRow.createCell(i);
            cell.setCellValue(headers.get(i));
            cell.setCellStyle(baseCellStyle);
            // 根据内容长度设置列宽
            int columnWidth = headers.get(i).length() * 256 * 2 + 256;
            sheet.setColumnWidth(i, columnWidth);
        }
        // 数据
        for (HouseBuildingPo po : list) {
            AtomicInteger columnIndex = new AtomicInteger(0);
            Row row = sheet.createRow(rowIndex.getAndIncrement());
            // 资产所属单位(一级)
            Cell firstUnitCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getFirstUnit() != null) {
                firstUnitCell.setCellValue(po.getFirstUnit());
            }
            // 二级单位
            Cell areaNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAreaName() != null) {
                areaNameCell.setCellValue(po.getAreaName());
            }
            // 三级单位
            Cell cityNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getCityName() != null) {
                cityNameCell.setCellValue(po.getCityName());
            }
            // 局址编号
            Cell siteNumCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getSiteName() != null) {
                siteNumCell.setCellValue(po.getSiteNum());
            }
            // 局址别名
            Cell siteNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getSiteName() != null) {
                siteNameCell.setCellValue(po.getSiteName());
            }
            // 建筑别名
            Cell buildingNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingName() != null) {
                buildingNameCell.setCellValue(po.getBuildingName());
            }
            // 地段
            Cell areaSectorCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAreaSector() != null) {
                areaSectorCell.setCellValue(po.getAreaSector());
            }
            // 建筑面积(㎡)
            Cell buildingAreaCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingArea() != null) {
                buildingAreaCell.setCellValue(po.getBuildingArea().doubleValue());
            }
            buildingAreaCell.setCellStyle(numberCellStyle);
            // 建筑面积-出租(㎡)
            Cell buildingAreaRentCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingAreaRent() != null) {
                buildingAreaRentCell.setCellValue(po.getBuildingAreaRent().doubleValue());
            }
            buildingAreaRentCell.setCellStyle(numberCellStyle);
            // 建筑面积-闲置(㎡)
            Cell buildingAreaIdleCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingAreaIdle() != null) {
                buildingAreaIdleCell.setCellValue(po.getBuildingAreaIdle().doubleValue());
            }
            buildingAreaIdleCell.setCellStyle(numberCellStyle);
            // 建筑面积-自用(㎡)
            Cell buildingAreaSelfUseCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingAreaSelfUse() != null) {
                buildingAreaSelfUseCell.setCellValue(po.getBuildingAreaSelfUse().doubleValue());
            }
            buildingAreaSelfUseCell.setCellStyle(numberCellStyle);
            // 建筑面积-不可使用(㎡)
            Cell buildingAreaUnusableCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingAreaUnusable() != null) {
                buildingAreaUnusableCell.setCellValue(po.getBuildingAreaUnusable().doubleValue());
            }
            buildingAreaUnusableCell.setCellStyle(numberCellStyle);
            // 上级土地名称
            Cell landNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getLandName() != null) {
                landNameCell.setCellValue(po.getLandName());
            }
            // 得房率
            Cell housingAcquisitionRateCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getHousingAcquisitionRate() != null) {
                housingAcquisitionRateCell.setCellValue(po.getHousingAcquisitionRate().doubleValue());
            }
            housingAcquisitionRateCell.setCellStyle(percentCellStyle);
            // 房屋来源
            Cell housingSourceCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getHousingSource() != null) {
                housingSourceCell.setCellValue(po.getHousingSource());
            }
            // 取得日期
            Cell acquisitionDateCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAcquisitionDate() != null) {
                acquisitionDateCell.setCellValue(po.getAcquisitionDate());
            }
            // 房龄开始年份
            Cell houseYearBeganCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getHouseYearBegan() != null) {
                houseYearBeganCell.setCellValue(po.getHouseYearBegan());
            }
            // 投资主体
            Cell investorCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getInvestor() != null) {
                investorCell.setCellValue(po.getInvestor());
            }
            // 管理层级
            Cell managementLevelCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getManagementLevel() != null) {
                managementLevelCell.setCellValue(po.getManagementLevel());
            }
            // 房屋结构
            Cell buildingStructureCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingStructure() != null) {
                buildingStructureCell.setCellValue(po.getBuildingStructure());
            }
            // 楼层总数
            Cell totalFloorsCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getTotalFloors() != null) {
                totalFloorsCell.setCellValue(po.getTotalFloors());
            }
            // 是否临街
            Cell frontageCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getFrontage() != null) {
                frontageCell.setCellValue(po.getFrontage());
            }
            // 是否有院落
            Cell courtyardCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getCourtyard() != null) {
                courtyardCell.setCellValue(po.getCourtyard());
            }
            // 整栋是否独有
            Cell wholeBuildingCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getWholeBuilding() != null) {
                wholeBuildingCell.setCellValue(po.getWholeBuilding());
            }
            // 是否有房产证
            Cell propertyOwnershipCertificateCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getPropertyOwnershipCertificate() != null) {
                propertyOwnershipCertificateCell.setCellValue(po.getPropertyOwnershipCertificate());
            }
            // 无房产证原因
            Cell noPropertyOwnershipCertificateReasonCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getNoPropertyOwnershipCertificateReason() != null) {
                noPropertyOwnershipCertificateReasonCell.setCellValue(
                        po.getNoPropertyOwnershipCertificateReason());
            }
            // 未关联资产
            Cell unrelatedAssetsCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUnrelatedAssets() != null) {
                unrelatedAssetsCell.setCellValue(po.getUnrelatedAssets());
            }
            // 资产编号
            Cell assetsNumCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAssetsNum() != null) {
                assetsNumCell.setCellValue(po.getAssetsNum());
            }
            // 资产标签号
            Cell assetsTagNumCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAssetsTagNum() != null) {
                assetsTagNumCell.setCellValue(po.getAssetsTagNum());
            }
            // 使用状态
            Cell usageStatusCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUsageStatus() != null) {
                usageStatusCell.setCellValue(po.getUsageStatus());
            }
            // 建筑用途
            Cell buildingUseCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingUse() != null) {
                buildingUseCell.setCellValue(po.getBuildingUse());
            }
            // 权属状态
            Cell ownershipStatusCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getOwnershipStatus() != null) {
                ownershipStatusCell.setCellValue(po.getOwnershipStatus());
            }
            // 建筑占地面积(㎡)
            Cell floorAreaCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getFloorArea() != null) {
                floorAreaCell.setCellValue(po.getFloorArea().doubleValue());
            }
            floorAreaCell.setCellStyle(numberCellStyle);
            // 使用面积(㎡)
            Cell usableAreaCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUsableArea() != null) {
                usableAreaCell.setCellValue(po.getUsableArea().doubleValue());
            }
            usableAreaCell.setCellStyle(numberCellStyle);
            // 使用面积-自用(㎡)
            Cell usableAreaSelfUseCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUsableAreaSelfUse() != null) {
                usableAreaSelfUseCell.setCellValue(po.getUsableAreaSelfUse().doubleValue());
            }
            usableAreaSelfUseCell.setCellStyle(numberCellStyle);
            // 使用面积-出租(㎡)
            Cell usableAreaRentCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUsableAreaRent() != null) {
                usableAreaRentCell.setCellValue(po.getUsableAreaRent().doubleValue());
            }
            usableAreaRentCell.setCellStyle(numberCellStyle);
            // 使用面积-闲置(㎡)
            Cell usableAreaIdleCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUsableAreaIdle() != null) {
                usableAreaIdleCell.setCellValue(po.getUsableAreaIdle().doubleValue());
            }
            usableAreaIdleCell.setCellStyle(numberCellStyle);
            // 使用面积-不可使用(㎡)
            Cell usableAreaUnusableCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUsableAreaUnusable() != null) {
                usableAreaUnusableCell.setCellValue(po.getUsableAreaUnusable().doubleValue());
            }
            usableAreaUnusableCell.setCellStyle(numberCellStyle);
            // 楼长姓名
            Cell communityAssistantNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getCommunityAssistantName() != null) {
                communityAssistantNameCell.setCellValue(po.getCommunityAssistantName());
            }
            // 楼长所在单位
            Cell communityAssistantUnitCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getCommunityAssistantUnit() != null) {
                communityAssistantUnitCell.setCellValue(po.getCommunityAssistantUnit());
            }
            // 经度
            Cell lngJtCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getLngJt() != null) {
                lngJtCell.setCellValue(po.getLngJt().toString());
            }
            // 纬度
            Cell latJtCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getLatJt() != null) {
                latJtCell.setCellValue(po.getLatJt().toString());
            }
            // 实际产权人
            Cell propertyOwnerCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getPropertyOwner() != null) {
                propertyOwnerCell.setCellValue(po.getPropertyOwner());
            }
        }
    }

    public Rsp<PageVo<String>> listSiteName(ListSiteNameDto dto) {
        Integer yearMonth = dto.getMap().getStatisticalMonth();
        if (yearMonth == null) {
            yearMonth = Integer.valueOf(LocalDate.now().minusDays(40)
                    .format(DateTimeFormatter.ofPattern("yyyyMM")));
        }
        if ("全省".equals(dto.getMap().getCity())) {
            dto.getMap().setCity(null);
            dto.getMap().setCounty(null);
        }
        ListHouseSiteBo bo = new ListHouseSiteBo();
        bo.setYearMonth(yearMonth);
        bo.setAreaNo(dto.getMap().getCity());
        bo.setCityNo(dto.getMap().getCounty());
        bo.setSiteName(dto.getMap().getAlias());
        Page<HouseSitePo> page = new Page<>(dto.getPage().getPageNum(), dto.getPage().getPageSize());
        List<HouseSitePo> list = wzOtnAreaDao.listHouseSite(page, bo);
        List<String> vos = list.stream().map(HouseSitePo::getSiteName).toList();
        PageVo<String> pageVo = new PageVo<>(vos, page);
        return Rsp.ok(pageVo);
    }

    public Rsp<PageVo<String>> listSiteNum(ListSiteNumDto dto) {
        Integer yearMonth = dto.getMap().getStatisticalMonth();
        if (yearMonth == null) {
            yearMonth = Integer.valueOf(LocalDate.now().minusDays(40)
                    .format(DateTimeFormatter.ofPattern("yyyyMM")));
        }
        if ("全省".equals(dto.getMap().getCity())) {
            dto.getMap().setCity(null);
            dto.getMap().setCounty(null);
        }
        ListHouseSiteBo bo = new ListHouseSiteBo();
        bo.setYearMonth(yearMonth);
        bo.setAreaNo(dto.getMap().getCity());
        bo.setCityNo(dto.getMap().getCounty());
        bo.setSiteNum(dto.getMap().getCode());
        Page<HouseSitePo> page = new Page<>(dto.getPage().getPageNum(), dto.getPage().getPageSize());
        List<HouseSitePo> list = wzOtnAreaDao.listHouseSite(page, bo);
        List<String> vos = list.stream().map(HouseSitePo::getSiteNum).toList();
        PageVo<String> pageVo = new PageVo<>(vos, page);
        return Rsp.ok(pageVo);
    }
}
