package com.example.service.house;

import cn.hutool.core.net.URLEncodeUtil;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.common.PageVo;
import com.example.common.Rsp;
import com.example.dao.WzOtnAreaDao;
import com.example.dao.house.HouseRepairExceptionDao;
import com.example.entity.house.HouseBuildingHighRepairPo;
import com.example.entity.house.HouseBuildingHighSporadicRepairPo;
import com.example.entity.house.HouseBuildingSameRepairFrequencyPo;
import com.example.entity.house.HouseSitePo;
import com.example.enums.ListBuildingHighRepairOrderEnum;
import com.example.enums.ListBuildingHighSporadicRepairOrderEnum;
import com.example.enums.ListBuildingSameRepairFrequencyOrderEnum;
import com.example.enums.OrderEnum;
import com.example.pojo.bo.ListBuildingHighRepairBo;
import com.example.pojo.bo.ListBuildingHighSporadicRepairBo;
import com.example.pojo.bo.ListBuildingSameRepairFrequencyBo;
import com.example.pojo.bo.ListHouseSiteBo;
import com.example.pojo.dto.ListBuildingHighRepairDto;
import com.example.pojo.dto.ListBuildingHighSporadicRepairDto;
import com.example.pojo.dto.ListBuildingSameRepairFrequencyDto;
import com.example.pojo.dto.ListSiteNameDto;
import com.example.pojo.dto.ListSiteNumDto;
import com.example.pojo.vo.ListBuildingHighRepairVo;
import com.example.pojo.vo.ListBuildingHighSporadicRepairVo;
import com.example.pojo.vo.ListBuildingSameRepairFrequencyVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

@Slf4j
@Service
public class HouseWzHouseMaintenaCostService {
    private final HouseRepairExceptionDao houseRepairExceptionDao;
    private final WzOtnAreaDao wzOtnAreaDao;

    public HouseWzHouseMaintenaCostService(HouseRepairExceptionDao houseRepairExceptionDao, WzOtnAreaDao wzOtnAreaDao) {
        this.houseRepairExceptionDao = houseRepairExceptionDao;
        this.wzOtnAreaDao = wzOtnAreaDao;
    }

    public Rsp<PageVo<ListBuildingHighRepairVo>> queryHighPrice(ListBuildingHighRepairDto dto) {
        Page<HouseBuildingHighRepairPo> page = new Page<>(dto.getPage().getPageNum(), dto.getPage().getPageSize());
        List<HouseBuildingHighRepairPo> list = getHouseBuildingHighRepairPos(dto, page);
        List<ListBuildingHighRepairVo> vos = new ArrayList<>();
        for (HouseBuildingHighRepairPo po : list) {
            vos.add(new ListBuildingHighRepairVo(po));
        }
        PageVo<ListBuildingHighRepairVo> pageVo = new PageVo<>(vos, page);
        return Rsp.ok(pageVo);
    }

    public void queryHighPriceExport(ListBuildingHighRepairDto dto) {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder
                .getRequestAttributes();
        if (servletRequestAttributes == null) {
            return;
        }
        HttpServletResponse response = servletRequestAttributes.getResponse();
        if (response == null) {
            return;
        }
        String filename = "维修异常监控_高总价统计"
                + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMdd")) + ".xlsx";
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncodeUtil.encode(filename));
        response.setContentType("application/octet-stream");
        try (SXSSFWorkbook wb = new SXSSFWorkbook()) {
            writeQueryHighPrice(dto, wb);
            wb.write(response.getOutputStream());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 查询不动产自有房产高总价统计
     */
    private List<HouseBuildingHighRepairPo> getHouseBuildingHighRepairPos(ListBuildingHighRepairDto dto,
                                                                          Page<HouseBuildingHighRepairPo> page) {
        ListBuildingHighRepairBo bo = new ListBuildingHighRepairBo();
        if (dto.getMap() != null && dto.getMap().getStatisticalMonth() != null) {
            bo.setYearMonth(dto.getMap().getStatisticalMonth());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCity()) && !"全省".equals(dto.getMap().getCity())) {
            bo.setAreaNo(dto.getMap().getCity());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCounty())) {
            bo.setCityNo(dto.getMap().getCounty());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getBuildingAddressNumber())) {
            bo.setSiteNum(dto.getMap().getBuildingAddressNumber());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getBuildingNameAlias())) {
            bo.setSiteName(dto.getMap().getBuildingNameAlias());
        }
        if (dto.getMap() != null && dto.getMap().getOrder() != null && dto.getMap().getSidx() != null) {
            if (OrderEnum.asc.equals(dto.getMap().getOrder())) {
                page.addOrder(OrderItem.asc(dto.getMap().getSidx().getColumnName()));
            } else {
                page.addOrder(OrderItem.desc(dto.getMap().getSidx().getColumnName()));
            }
        } else {
            page.addOrder(OrderItem.desc(ListBuildingHighRepairOrderEnum.erpCreditedAmount.getColumnName()));
        }
        return houseRepairExceptionDao.listBuildingHighRepair(page, bo);
    }

    /**
     * 写不动产自有房产高总价统计文件
     */
    private void writeQueryHighPrice(ListBuildingHighRepairDto dto, SXSSFWorkbook wb) {
        Page<HouseBuildingHighRepairPo> page = new Page<>(1, Long.MAX_VALUE);
        List<HouseBuildingHighRepairPo> list = getHouseBuildingHighRepairPos(dto, page);
        DataFormat dataFormat = wb.createDataFormat();
        // 数字样式
        CellStyle numberCellStyle = wb.createCellStyle();
        numberCellStyle.setDataFormat(dataFormat.getFormat("#,##0.00"));
        numberCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        // 默认样式
        CellStyle baseCellStyle = wb.createCellStyle();
        baseCellStyle.setAlignment(HorizontalAlignment.CENTER);
        baseCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        SXSSFSheet sheet = wb.createSheet();
        AtomicInteger rowIndex = new AtomicInteger(0);
        // 表头
        Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
        List<String> headers = Stream.of("入账日期", "二级单位", "三级单位", "局址编号", "局址别名", "建筑别名",
                "维修类型", "维修内容", "erp入账金额元", "标准地址", "建筑用途", "建筑面积㎡", "建筑面积自用㎡", "建筑面积出租㎡",
                "建筑面积闲置㎡", "建筑面积不可使用㎡").toList();
        int headerLength = headers.size();
        for (int i = 0; i < headerLength; i++) {
            Cell cell = headerRow.createCell(i);
            cell.setCellValue(headers.get(i));
            cell.setCellStyle(baseCellStyle);
            // 根据内容长度设置列宽
            int columnWidth = headers.get(i).length() * 256 * 2 + 256;
            sheet.setColumnWidth(i, columnWidth);
        }
        // 数据
        for (HouseBuildingHighRepairPo po : list) {
            AtomicInteger columnIndex = new AtomicInteger(0);
            Row row = sheet.createRow(rowIndex.getAndIncrement());
            // 入账日期
            Cell latestRepairMonthCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getLatestRepairMonth() != null) {
                latestRepairMonthCell.setCellValue(po.getLatestRepairMonth());
            }
            // 二级单位
            Cell areaNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAreaName() != null) {
                areaNameCell.setCellValue(po.getAreaName());
            }
            // 三级单位
            Cell cityNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getCityName() != null) {
                cityNameCell.setCellValue(po.getCityName());
            }
            // 局址编号
            Cell siteNumCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getSiteName() != null) {
                siteNumCell.setCellValue(po.getSiteNum());
            }
            // 局址别名
            Cell siteNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getSiteName() != null) {
                siteNameCell.setCellValue(po.getSiteName());
            }
            // 建筑别名
            Cell buildingNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingName() != null) {
                buildingNameCell.setCellValue(po.getBuildingName());
            }
            // 维修类型
            Cell repairTypeCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getRepairType() != null) {
                repairTypeCell.setCellValue(po.getRepairType());
            }
            // 维修内容
            Cell contentCategoryCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getContentCategory() != null) {
                contentCategoryCell.setCellValue(po.getContentCategory());
            }
            // erp入账金额元
            Cell finalCostSumCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getFinalCostSum() != null) {
                finalCostSumCell.setCellValue(po.getFinalCostSum().doubleValue());
            }
            finalCostSumCell.setCellStyle(numberCellStyle);
            // 标准地址
            Cell addressCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAddress() != null) {
                addressCell.setCellValue(po.getAddress());
            }
            // 建筑用途
            Cell buildingUseCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingUse() != null) {
                buildingUseCell.setCellValue(po.getBuildingUse());
            }
            // 建筑面积㎡
            Cell buildingAreaCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingArea() != null) {
                buildingAreaCell.setCellValue(po.getBuildingArea().doubleValue());
            }
            buildingAreaCell.setCellStyle(numberCellStyle);
            // 建筑面积自用㎡
            Cell buildingAreaSelfUseCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingAreaSelfUse() != null) {
                buildingAreaSelfUseCell.setCellValue(po.getBuildingAreaSelfUse().doubleValue());
            }
            buildingAreaSelfUseCell.setCellStyle(numberCellStyle);
            // 建筑面积出租㎡
            Cell buildingAreaRentCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingAreaRent() != null) {
                buildingAreaRentCell.setCellValue(po.getBuildingAreaRent().doubleValue());
            }
            buildingAreaRentCell.setCellStyle(numberCellStyle);
            // 建筑面积闲置㎡
            Cell buildingAreaIdleCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingAreaIdle() != null) {
                buildingAreaIdleCell.setCellValue(po.getBuildingAreaIdle().doubleValue());
            }
            buildingAreaIdleCell.setCellStyle(numberCellStyle);
            // 建筑面积不可使用㎡
            Cell buildingAreaUnusableCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingAreaUnusable() != null) {
                buildingAreaUnusableCell.setCellValue(po.getBuildingAreaUnusable().doubleValue());
            }
            buildingAreaUnusableCell.setCellStyle(numberCellStyle);
        }
    }

    public Rsp<PageVo<ListBuildingSameRepairFrequencyVo>> queryHighFrequency(ListBuildingSameRepairFrequencyDto dto) {
        Page<HouseBuildingSameRepairFrequencyPo> page = new Page<>(dto.getPage().getPageNum(), dto.getPage().getPageSize());
        List<HouseBuildingSameRepairFrequencyPo> list = getHouseBuildingSameRepairFrequencyPos(dto, page);
        List<ListBuildingSameRepairFrequencyVo> vos = new ArrayList<>();
        for (HouseBuildingSameRepairFrequencyPo po : list) {
            vos.add(new ListBuildingSameRepairFrequencyVo(po));
        }
        PageVo<ListBuildingSameRepairFrequencyVo> pageVo = new PageVo<>(vos, page);
        return Rsp.ok(pageVo);
    }

    /**
     * 查询不动产自有房产相同维修频次统计
     */
    private List<HouseBuildingSameRepairFrequencyPo> getHouseBuildingSameRepairFrequencyPos(
            ListBuildingSameRepairFrequencyDto dto,
            Page<HouseBuildingSameRepairFrequencyPo> page) {
        ListBuildingSameRepairFrequencyBo bo = new ListBuildingSameRepairFrequencyBo();
        if (dto.getMap() != null && dto.getMap().getStatisticalMonth() != null) {
            bo.setYearMonth(dto.getMap().getStatisticalMonth());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCity()) && !"全省".equals(dto.getMap().getCity())) {
            bo.setAreaNo(dto.getMap().getCity());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCounty())) {
            bo.setCityNo(dto.getMap().getCounty());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getBuildingAddressNumber())) {
            bo.setSiteNum(dto.getMap().getBuildingAddressNumber());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getBuildingNameAlias())) {
            bo.setSiteName(dto.getMap().getBuildingNameAlias());
        }
        if (dto.getMap() != null && dto.getMap().getOrder() != null && dto.getMap().getSidx() != null) {
            if (OrderEnum.asc.equals(dto.getMap().getOrder())) {
                page.addOrder(OrderItem.asc(dto.getMap().getSidx().getColumnName()));
            } else {
                page.addOrder(OrderItem.desc(dto.getMap().getSidx().getColumnName()));
            }
        } else {
            page.addOrder(OrderItem.asc(ListBuildingSameRepairFrequencyOrderEnum.tyTotalRank.getColumnName()));
        }
        return houseRepairExceptionDao.listBuildingSameRepairFrequency(page, bo);
    }

    public void queryHighFrequencyExport(ListBuildingSameRepairFrequencyDto dto) {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder
                .getRequestAttributes();
        if (servletRequestAttributes == null) {
            return;
        }
        HttpServletResponse response = servletRequestAttributes.getResponse();
        if (response == null) {
            return;
        }
        String filename = "维修异常监控_相同维修频次统计"
                + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMdd")) + ".xlsx";
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncodeUtil.encode(filename));
        response.setContentType("application/octet-stream");
        try (SXSSFWorkbook wb = new SXSSFWorkbook()) {
            writeQueryHighFrequency(dto, wb);
            wb.write(response.getOutputStream());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 写不动产自有房产相同维修频次统计文件
     */
    private void writeQueryHighFrequency(ListBuildingSameRepairFrequencyDto dto, SXSSFWorkbook wb) {
        Page<HouseBuildingSameRepairFrequencyPo> page = new Page<>(1, Long.MAX_VALUE);
        List<HouseBuildingSameRepairFrequencyPo> list = getHouseBuildingSameRepairFrequencyPos(dto, page);
        DataFormat dataFormat = wb.createDataFormat();
        // 数字样式
        CellStyle numberCellStyle = wb.createCellStyle();
        numberCellStyle.setDataFormat(dataFormat.getFormat("#,##0"));
        numberCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        // 默认样式
        CellStyle baseCellStyle = wb.createCellStyle();
        baseCellStyle.setAlignment(HorizontalAlignment.CENTER);
        baseCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        SXSSFSheet sheet = wb.createSheet();
        AtomicInteger rowIndex = new AtomicInteger(0);
        // 表头
        Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
        List<String> headers = Stream.of("局址编号", "局址别名", "单位", "维修建筑", "维修类别", "维修内容",
                "账期时间", "本年累计发生频率", "排名", "202301至今累计发生频率").toList();
        int headerLength = headers.size();
        for (int i = 0; i < headerLength; i++) {
            Cell cell = headerRow.createCell(i);
            cell.setCellValue(headers.get(i));
            cell.setCellStyle(baseCellStyle);
            // 根据内容长度设置列宽
            int columnWidth = headers.get(i).length() * 256 * 2 + 256;
            sheet.setColumnWidth(i, columnWidth);
        }
        // 数据
        for (HouseBuildingSameRepairFrequencyPo po : list) {
            AtomicInteger columnIndex = new AtomicInteger(0);
            Row row = sheet.createRow(rowIndex.getAndIncrement());
            // 局址编号
            Cell siteNumCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getSiteName() != null) {
                siteNumCell.setCellValue(po.getSiteNum());
            }
            // 局址别名
            Cell siteNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getSiteName() != null) {
                siteNameCell.setCellValue(po.getSiteName());
            }
            // 二级单位
            Cell areaNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAreaName() != null) {
                areaNameCell.setCellValue(po.getAreaName());
            }
            // 维修建筑
            Cell buildingNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingName() != null) {
                buildingNameCell.setCellValue(po.getBuildingName());
            }
            // 维修类别
            Cell repairTypeCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getRepairType() != null) {
                repairTypeCell.setCellValue(po.getRepairType());
            }
            // 维修内容
            Cell contentCategoryCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getContentCategory() != null) {
                contentCategoryCell.setCellValue(po.getContentCategory());
            }
            // 账期时间
            Cell yearMonthCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getYearMonth() != null) {
                yearMonthCell.setCellValue(po.getYearMonth());
            }
            // 本年累计发生频率
            Cell tyTotalCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getTyTotal() != null) {
                tyTotalCell.setCellValue(po.getTyTotal());
            }
            tyTotalCell.setCellStyle(numberCellStyle);
            // 排名
            Cell tyTotalRankCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getTyTotalRank() != null) {
                tyTotalRankCell.setCellValue(po.getTyTotalRank());
            }
            // 202301至今累计发生频率
            Cell historyTotalCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getHistoryTotal() != null) {
                historyTotalCell.setCellValue(po.getHistoryTotal());
            }
            historyTotalCell.setCellStyle(numberCellStyle);
        }
    }

    public Rsp<PageVo<ListBuildingHighSporadicRepairVo>> queryDailyModify(ListBuildingHighSporadicRepairDto dto) {
        Page<HouseBuildingHighSporadicRepairPo> page = new Page<>(dto.getPage().getPageNum(), dto.getPage().getPageSize());
        List<HouseBuildingHighSporadicRepairPo> list = getHouseBuildingHighSporadicRepairPos(dto, page);
        List<ListBuildingHighSporadicRepairVo> vos = new ArrayList<>();
        for (HouseBuildingHighSporadicRepairPo po : list) {
            vos.add(new ListBuildingHighSporadicRepairVo(po));
        }
        PageVo<ListBuildingHighSporadicRepairVo> pageVo = new PageVo<>(vos, page);
        return Rsp.ok(pageVo);
    }

    /**
     * 查询不动产自有房产高额零星维修统计
     */
    private List<HouseBuildingHighSporadicRepairPo> getHouseBuildingHighSporadicRepairPos(
            ListBuildingHighSporadicRepairDto dto, Page<HouseBuildingHighSporadicRepairPo> page) {
        ListBuildingHighSporadicRepairBo bo = new ListBuildingHighSporadicRepairBo();
        if (dto.getMap() != null && dto.getMap().getStatisticalMonth() != null) {
            bo.setYearMonth(dto.getMap().getStatisticalMonth());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCity()) && !"全省".equals(dto.getMap().getCity())) {
            bo.setAreaNo(dto.getMap().getCity());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCounty())) {
            bo.setCityNo(dto.getMap().getCounty());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getBuildingAddressNumber())) {
            bo.setSiteNum(dto.getMap().getBuildingAddressNumber());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getBuildingNameAlias())) {
            bo.setSiteName(dto.getMap().getBuildingNameAlias());
        }
        if (dto.getMap() != null && dto.getMap().getOrder() != null && dto.getMap().getSidx() != null) {
            if (OrderEnum.asc.equals(dto.getMap().getOrder())) {
                page.addOrder(OrderItem.asc(dto.getMap().getSidx().getColumnName()));
            } else {
                page.addOrder(OrderItem.desc(dto.getMap().getSidx().getColumnName()));
            }
        } else {
            page.addOrder(OrderItem.desc(ListBuildingHighSporadicRepairOrderEnum.erpMoney.getColumnName()));
        }
        return houseRepairExceptionDao.ListBuildingHighSporadicRepair(page, bo);
    }

    public void queryDailyModifyExport(ListBuildingHighSporadicRepairDto dto) {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder
                .getRequestAttributes();
        if (servletRequestAttributes == null) {
            return;
        }
        HttpServletResponse response = servletRequestAttributes.getResponse();
        if (response == null) {
            return;
        }
        String filename = "维修异常监控_高额日常零星维修"
                + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMdd")) + ".xlsx";
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncodeUtil.encode(filename));
        response.setContentType("application/octet-stream");
        try (SXSSFWorkbook wb = new SXSSFWorkbook()) {
            writeQueryDailyModify(dto, wb);
            wb.write(response.getOutputStream());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 写高额日常零星维修文件
     */
    private void writeQueryDailyModify(ListBuildingHighSporadicRepairDto dto, SXSSFWorkbook wb) {
        Page<HouseBuildingHighSporadicRepairPo> page = new Page<>(1, Long.MAX_VALUE);
        List<HouseBuildingHighSporadicRepairPo> list = getHouseBuildingHighSporadicRepairPos(dto, page);
        DataFormat dataFormat = wb.createDataFormat();
        // 数字样式
        CellStyle numberCellStyle = wb.createCellStyle();
        numberCellStyle.setDataFormat(dataFormat.getFormat("#,##0.00"));
        numberCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        // 默认样式
        CellStyle baseCellStyle = wb.createCellStyle();
        baseCellStyle.setAlignment(HorizontalAlignment.CENTER);
        baseCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        SXSSFSheet sheet = wb.createSheet();
        AtomicInteger rowIndex = new AtomicInteger(0);
        // 表头
        Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
        List<String> headers = Stream.of("单位", "维修建筑", "维修时间", "局址编号",
                "局址别名", "维修类型", "维修内容", "维修金额", "本年超万元次数").toList();
        int headerLength = headers.size();
        for (int i = 0; i < headerLength; i++) {
            Cell cell = headerRow.createCell(i);
            cell.setCellValue(headers.get(i));
            cell.setCellStyle(baseCellStyle);
            // 根据内容长度设置列宽
            int columnWidth = headers.get(i).length() * 256 * 2 + 256;
            sheet.setColumnWidth(i, columnWidth);
        }
        // 数据
        for (HouseBuildingHighSporadicRepairPo po : list) {
            AtomicInteger columnIndex = new AtomicInteger(0);
            Row row = sheet.createRow(rowIndex.getAndIncrement());
            // 二级单位
            Cell areaNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAreaName() != null) {
                areaNameCell.setCellValue(po.getAreaName());
            }
            // 维修建筑
            Cell buildingNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingName() != null) {
                buildingNameCell.setCellValue(po.getBuildingName());
            }
            // 维修时间
            Cell repairYearMonthCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getRepairYearMonth() != null) {
                repairYearMonthCell.setCellValue(po.getRepairYearMonth());
            }
            // 局址编号
            Cell siteNumCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getSiteName() != null) {
                siteNumCell.setCellValue(po.getSiteNum());
            }
            // 局址别名
            Cell siteNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getSiteName() != null) {
                siteNameCell.setCellValue(po.getSiteName());
            }
            // 维修类型
            Cell repairTypeCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getRepairType() != null) {
                repairTypeCell.setCellValue(po.getRepairType());
            }
            // 维修内容
            Cell repairContentCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getRepairContent() != null) {
                repairContentCell.setCellValue(po.getRepairContent());
            }
            // 维修金额
            Cell amountCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAmount() != null) {
                amountCell.setCellValue(po.getAmount().doubleValue());
            }
            amountCell.setCellStyle(numberCellStyle);
            // 本年超万元次数
            Cell totalCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getTotal() != null) {
                totalCell.setCellValue(po.getTotal());
            }
        }
    }

    public Rsp<PageVo<String>> listSiteName(ListSiteNameDto dto) {
        Integer yearMonth = dto.getMap().getStatisticalMonth();
        if (yearMonth == null) {
            yearMonth = Integer.valueOf(LocalDate.now().minusDays(40)
                    .format(DateTimeFormatter.ofPattern("yyyyMM")));
        }
        if ("全省".equals(dto.getMap().getCity())) {
            dto.getMap().setCity(null);
            dto.getMap().setCounty(null);
        }
        ListHouseSiteBo bo = new ListHouseSiteBo();
        bo.setYearMonth(yearMonth);
        bo.setAreaNo(dto.getMap().getCity());
        bo.setCityNo(dto.getMap().getCounty());
        bo.setSiteName(dto.getMap().getAlias());
        Page<HouseSitePo> page = new Page<>(dto.getPage().getPageNum(), dto.getPage().getPageSize());
        List<HouseSitePo> list = wzOtnAreaDao.listHouseSite(page, bo);
        List<String> vos = list.stream().map(HouseSitePo::getSiteName).toList();
        PageVo<String> pageVo = new PageVo<>(vos, page);
        return Rsp.ok(pageVo);
    }

    public Rsp<PageVo<String>> listSiteNum(ListSiteNumDto dto) {
        Integer yearMonth = dto.getMap().getStatisticalMonth();
        if (yearMonth == null) {
            yearMonth = Integer.valueOf(LocalDate.now().minusDays(40)
                    .format(DateTimeFormatter.ofPattern("yyyyMM")));
        }
        if ("全省".equals(dto.getMap().getCity())) {
            dto.getMap().setCity(null);
            dto.getMap().setCounty(null);
        }
        ListHouseSiteBo bo = new ListHouseSiteBo();
        bo.setYearMonth(yearMonth);
        bo.setAreaNo(dto.getMap().getCity());
        bo.setCityNo(dto.getMap().getCounty());
        bo.setSiteNum(dto.getMap().getCode());
        Page<HouseSitePo> page = new Page<>(dto.getPage().getPageNum(), dto.getPage().getPageSize());
        List<HouseSitePo> list = wzOtnAreaDao.listHouseSite(page, bo);
        List<String> vos = list.stream().map(HouseSitePo::getSiteNum).toList();
        PageVo<String> pageVo = new PageVo<>(vos, page);
        return Rsp.ok(pageVo);
    }
}
